use std::path::Path;
use std::{fmt::Display, path::PathBuf};

use crate::error::Error;

use super::{Section, Symbol};

#[derive(Clone, Copy)]
// index into section of given object stored in the linker
pub struct DataIndex {
    pub object_index: usize,
    pub section_index: usize,
}

impl DataIndex {
    pub fn new(object_index: usize, section_index: usize) -> Self {
        DataIndex {
            object_index,
            section_index,
        }
    }

    pub fn for_object(object_index: usize) -> Self {
        DataIndex {
            object_index,
            section_index: 0,
        }
    }
}

pub type SectionIterBox<'iter> = Box<dyn Iterator<Item = Result<Section, Error>> + 'iter>;

pub type SymbolIterBox<'iter> = Box<dyn Iterator<Item = Symbol> + 'iter>;

/// Contains all the needed getters to construct a final
/// mushed and relocated executable from an object file
pub trait Relocatable: Display + Sized {
    // + TryFrom<PathBuf, Error = Error> {
    fn new(origin: PathBuf, index: DataIndex) -> Result<Self, Error>;

    fn origin(&self) -> &Path; // not same as section's path since this one's supposed to be cannonical

    fn sections(&self) -> SectionIterBox;

    fn section_data(&self, section_index: DataIndex) -> Result<&[u8], Error>;

    fn symbols(&self) -> SymbolIterBox;
}
